sqlglot.dialects.postgres API documentation

Edit on GitHub

# sqlglot.dialects.postgres

View Source

      1from __future__ import annotations
      2
      3import typing as t
      4
      5from sqlglot import exp, generator, parser, tokens, transforms
      6from sqlglot.dialects.dialect import (
      7    Dialect,
      8    arrow_json_extract_scalar_sql,
      9    arrow_json_extract_sql,
     10    datestrtodate_sql,
     11    format_time_lambda,
     12    max_or_greatest,
     13    min_or_least,
     14    no_map_from_entries_sql,
     15    no_paren_current_date_sql,
     16    no_pivot_sql,
     17    no_tablesample_sql,
     18    no_trycast_sql,
     19    rename_func,
     20    str_position_sql,
     21    timestamptrunc_sql,
     22    timestrtotime_sql,
     23    trim_sql,
     24    ts_or_ds_to_date_sql,
     25)
     26from sqlglot.helper import seq_get
     27from sqlglot.parser import binary_range_parser
     28from sqlglot.tokens import TokenType
     29
     30DATE_DIFF_FACTOR = {
     31    "MICROSECOND": " * 1000000",
     32    "MILLISECOND": " * 1000",
     33    "SECOND": "",
     34    "MINUTE": " / 60",
     35    "HOUR": " / 3600",
     36    "DAY": " / 86400",
     37}
     38
     39
     40def _date_add_sql(kind: str) -> t.Callable[[generator.Generator, exp.DateAdd | exp.DateSub], str]:
     41    def func(self: generator.Generator, expression: exp.DateAdd | exp.DateSub) -> str:
     42        from sqlglot.optimizer.simplify import simplify
     43
     44        this = self.sql(expression, "this")
     45        unit = expression.args.get("unit")
     46        expression = simplify(expression.args["expression"])
     47
     48        if not isinstance(expression, exp.Literal):
     49            self.unsupported("Cannot add non literal")
     50
     51        expression = expression.copy()
     52        expression.args["is_string"] = True
     53        return f"{this} {kind} {self.sql(exp.Interval(this=expression, unit=unit))}"
     54
     55    return func
     56
     57
     58def _date_diff_sql(self: generator.Generator, expression: exp.DateDiff) -> str:
     59    unit = expression.text("unit").upper()
     60    factor = DATE_DIFF_FACTOR.get(unit)
     61
     62    end = f"CAST({expression.this} AS TIMESTAMP)"
     63    start = f"CAST({expression.expression} AS TIMESTAMP)"
     64
     65    if factor is not None:
     66        return f"CAST(EXTRACT(epoch FROM {end} - {start}){factor} AS BIGINT)"
     67
     68    age = f"AGE({end}, {start})"
     69
     70    if unit == "WEEK":
     71        unit = f"EXTRACT(year FROM {age}) * 48 + EXTRACT(month FROM {age}) * 4 + EXTRACT(day FROM {age}) / 7"
     72    elif unit == "MONTH":
     73        unit = f"EXTRACT(year FROM {age}) * 12 + EXTRACT(month FROM {age})"
     74    elif unit == "QUARTER":
     75        unit = f"EXTRACT(year FROM {age}) * 4 + EXTRACT(month FROM {age}) / 3"
     76    elif unit == "YEAR":
     77        unit = f"EXTRACT(year FROM {age})"
     78    else:
     79        unit = age
     80
     81    return f"CAST({unit} AS BIGINT)"
     82
     83
     84def _substring_sql(self: generator.Generator, expression: exp.Substring) -> str:
     85    this = self.sql(expression, "this")
     86    start = self.sql(expression, "start")
     87    length = self.sql(expression, "length")
     88
     89    from_part = f" FROM {start}" if start else ""
     90    for_part = f" FOR {length}" if length else ""
     91
     92    return f"SUBSTRING({this}{from_part}{for_part})"
     93
     94
     95def _string_agg_sql(self: generator.Generator, expression: exp.GroupConcat) -> str:
     96    expression = expression.copy()
     97    separator = expression.args.get("separator") or exp.Literal.string(",")
     98
     99    order = ""
    100    this = expression.this
    101    if isinstance(this, exp.Order):
    102        if this.this:
    103            this = this.this.pop()
    104        order = self.sql(expression.this)  # Order has a leading space
    105
    106    return f"STRING_AGG({self.format_args(this, separator)}{order})"
    107
    108
    109def _datatype_sql(self: generator.Generator, expression: exp.DataType) -> str:
    110    if expression.is_type("array"):
    111        return f"{self.expressions(expression, flat=True)}[]"
    112    return self.datatype_sql(expression)
    113
    114
    115def _auto_increment_to_serial(expression: exp.Expression) -> exp.Expression:
    116    auto = expression.find(exp.AutoIncrementColumnConstraint)
    117
    118    if auto:
    119        expression = expression.copy()
    120        expression.args["constraints"].remove(auto.parent)
    121        kind = expression.args["kind"]
    122
    123        if kind.this == exp.DataType.Type.INT:
    124            kind.replace(exp.DataType(this=exp.DataType.Type.SERIAL))
    125        elif kind.this == exp.DataType.Type.SMALLINT:
    126            kind.replace(exp.DataType(this=exp.DataType.Type.SMALLSERIAL))
    127        elif kind.this == exp.DataType.Type.BIGINT:
    128            kind.replace(exp.DataType(this=exp.DataType.Type.BIGSERIAL))
    129
    130    return expression
    131
    132
    133def _serial_to_generated(expression: exp.Expression) -> exp.Expression:
    134    kind = expression.args["kind"]
    135
    136    if kind.this == exp.DataType.Type.SERIAL:
    137        data_type = exp.DataType(this=exp.DataType.Type.INT)
    138    elif kind.this == exp.DataType.Type.SMALLSERIAL:
    139        data_type = exp.DataType(this=exp.DataType.Type.SMALLINT)
    140    elif kind.this == exp.DataType.Type.BIGSERIAL:
    141        data_type = exp.DataType(this=exp.DataType.Type.BIGINT)
    142    else:
    143        data_type = None
    144
    145    if data_type:
    146        expression = expression.copy()
    147        expression.args["kind"].replace(data_type)
    148        constraints = expression.args["constraints"]
    149        generated = exp.ColumnConstraint(kind=exp.GeneratedAsIdentityColumnConstraint(this=False))
    150        notnull = exp.ColumnConstraint(kind=exp.NotNullColumnConstraint())
    151
    152        if notnull not in constraints:
    153            constraints.insert(0, notnull)
    154        if generated not in constraints:
    155            constraints.insert(0, generated)
    156
    157    return expression
    158
    159
    160def _generate_series(args: t.List) -> exp.Expression:
    161    # The goal is to convert step values like '1 day' or INTERVAL '1 day' into INTERVAL '1' day
    162    step = seq_get(args, 2)
    163
    164    if step is None:
    165        # Postgres allows calls with just two arguments -- the "step" argument defaults to 1
    166        return exp.GenerateSeries.from_arg_list(args)
    167
    168    if step.is_string:
    169        args[2] = exp.to_interval(step.this)
    170    elif isinstance(step, exp.Interval) and not step.args.get("unit"):
    171        args[2] = exp.to_interval(step.this.this)
    172
    173    return exp.GenerateSeries.from_arg_list(args)
    174
    175
    176def _to_timestamp(args: t.List) -> exp.Expression:
    177    # TO_TIMESTAMP accepts either a single double argument or (text, text)
    178    if len(args) == 1:
    179        # https://www.postgresql.org/docs/current/functions-datetime.html#FUNCTIONS-DATETIME-TABLE
    180        return exp.UnixToTime.from_arg_list(args)
    181
    182    # https://www.postgresql.org/docs/current/functions-formatting.html
    183    return format_time_lambda(exp.StrToTime, "postgres")(args)
    184
    185
    186class Postgres(Dialect):
    187    INDEX_OFFSET = 1
    188    NULL_ORDERING = "nulls_are_large"
    189    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    190    TIME_MAPPING = {
    191        "AM": "%p",
    192        "PM": "%p",
    193        "D": "%u",  # 1-based day of week
    194        "DD": "%d",  # day of month
    195        "DDD": "%j",  # zero padded day of year
    196        "FMDD": "%-d",  # - is no leading zero for Python; same for FM in postgres
    197        "FMDDD": "%-j",  # day of year
    198        "FMHH12": "%-I",  # 9
    199        "FMHH24": "%-H",  # 9
    200        "FMMI": "%-M",  # Minute
    201        "FMMM": "%-m",  # 1
    202        "FMSS": "%-S",  # Second
    203        "HH12": "%I",  # 09
    204        "HH24": "%H",  # 09
    205        "MI": "%M",  # zero padded minute
    206        "MM": "%m",  # 01
    207        "OF": "%z",  # utc offset
    208        "SS": "%S",  # zero padded second
    209        "TMDay": "%A",  # TM is locale dependent
    210        "TMDy": "%a",
    211        "TMMon": "%b",  # Sep
    212        "TMMonth": "%B",  # September
    213        "TZ": "%Z",  # uppercase timezone name
    214        "US": "%f",  # zero padded microsecond
    215        "WW": "%U",  # 1-based week of year
    216        "YY": "%y",  # 15
    217        "YYYY": "%Y",  # 2015
    218    }
    219
    220    class Tokenizer(tokens.Tokenizer):
    221        QUOTES = ["'", "$$"]
    222
    223        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    224        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    225        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    226
    227        KEYWORDS = {
    228            **tokens.Tokenizer.KEYWORDS,
    229            "~~": TokenType.LIKE,
    230            "~~*": TokenType.ILIKE,
    231            "~*": TokenType.IRLIKE,
    232            "~": TokenType.RLIKE,
    233            "@>": TokenType.AT_GT,
    234            "<@": TokenType.LT_AT,
    235            "BEGIN": TokenType.COMMAND,
    236            "BEGIN TRANSACTION": TokenType.BEGIN,
    237            "BIGSERIAL": TokenType.BIGSERIAL,
    238            "CHARACTER VARYING": TokenType.VARCHAR,
    239            "DECLARE": TokenType.COMMAND,
    240            "DO": TokenType.COMMAND,
    241            "HSTORE": TokenType.HSTORE,
    242            "JSONB": TokenType.JSONB,
    243            "REFRESH": TokenType.COMMAND,
    244            "REINDEX": TokenType.COMMAND,
    245            "RESET": TokenType.COMMAND,
    246            "REVOKE": TokenType.COMMAND,
    247            "SERIAL": TokenType.SERIAL,
    248            "SMALLSERIAL": TokenType.SMALLSERIAL,
    249            "TEMP": TokenType.TEMPORARY,
    250            "CSTRING": TokenType.PSEUDO_TYPE,
    251        }
    252
    253        SINGLE_TOKENS = {
    254            **tokens.Tokenizer.SINGLE_TOKENS,
    255            "$": TokenType.PARAMETER,
    256        }
    257
    258        VAR_SINGLE_TOKENS = {"$"}
    259
    260    class Parser(parser.Parser):
    261        STRICT_CAST = False
    262        CONCAT_NULL_OUTPUTS_STRING = True
    263
    264        FUNCTIONS = {
    265            **parser.Parser.FUNCTIONS,
    266            "DATE_TRUNC": lambda args: exp.TimestampTrunc(
    267                this=seq_get(args, 1), unit=seq_get(args, 0)
    268            ),
    269            "GENERATE_SERIES": _generate_series,
    270            "NOW": exp.CurrentTimestamp.from_arg_list,
    271            "TO_CHAR": format_time_lambda(exp.TimeToStr, "postgres"),
    272            "TO_TIMESTAMP": _to_timestamp,
    273            "UNNEST": exp.Explode.from_arg_list,
    274        }
    275
    276        FUNCTION_PARSERS = {
    277            **parser.Parser.FUNCTION_PARSERS,
    278            "DATE_PART": lambda self: self._parse_date_part(),
    279        }
    280
    281        BITWISE = {
    282            **parser.Parser.BITWISE,
    283            TokenType.HASH: exp.BitwiseXor,
    284        }
    285
    286        EXPONENT = {
    287            TokenType.CARET: exp.Pow,
    288        }
    289
    290        RANGE_PARSERS = {
    291            **parser.Parser.RANGE_PARSERS,
    292            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    293            TokenType.AT_GT: binary_range_parser(exp.ArrayContains),
    294            TokenType.LT_AT: binary_range_parser(exp.ArrayContained),
    295        }
    296
    297        def _parse_factor(self) -> t.Optional[exp.Expression]:
    298            return self._parse_tokens(self._parse_exponent, self.FACTOR)
    299
    300        def _parse_exponent(self) -> t.Optional[exp.Expression]:
    301            return self._parse_tokens(self._parse_unary, self.EXPONENT)
    302
    303        def _parse_date_part(self) -> exp.Expression:
    304            part = self._parse_type()
    305            self._match(TokenType.COMMA)
    306            value = self._parse_bitwise()
    307
    308            if part and part.is_string:
    309                part = exp.var(part.name)
    310
    311            return self.expression(exp.Extract, this=part, expression=value)
    312
    313    class Generator(generator.Generator):
    314        SINGLE_STRING_INTERVAL = True
    315        LOCKING_READS_SUPPORTED = True
    316        JOIN_HINTS = False
    317        TABLE_HINTS = False
    318        PARAMETER_TOKEN = "$"
    319
    320        TYPE_MAPPING = {
    321            **generator.Generator.TYPE_MAPPING,
    322            exp.DataType.Type.TINYINT: "SMALLINT",
    323            exp.DataType.Type.FLOAT: "REAL",
    324            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    325            exp.DataType.Type.BINARY: "BYTEA",
    326            exp.DataType.Type.VARBINARY: "BYTEA",
    327            exp.DataType.Type.DATETIME: "TIMESTAMP",
    328        }
    329
    330        TRANSFORMS = {
    331            **generator.Generator.TRANSFORMS,
    332            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    333            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    334            exp.Explode: rename_func("UNNEST"),
    335            exp.JSONExtract: arrow_json_extract_sql,
    336            exp.JSONExtractScalar: arrow_json_extract_scalar_sql,
    337            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    338            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    339            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    340            exp.Pow: lambda self, e: self.binary(e, "^"),
    341            exp.CurrentDate: no_paren_current_date_sql,
    342            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    343            exp.DateAdd: _date_add_sql("+"),
    344            exp.DateStrToDate: datestrtodate_sql,
    345            exp.DateSub: _date_add_sql("-"),
    346            exp.DateDiff: _date_diff_sql,
    347            exp.LogicalOr: rename_func("BOOL_OR"),
    348            exp.LogicalAnd: rename_func("BOOL_AND"),
    349            exp.Max: max_or_greatest,
    350            exp.MapFromEntries: no_map_from_entries_sql,
    351            exp.Min: min_or_least,
    352            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    353            exp.ArrayContains: lambda self, e: self.binary(e, "@>"),
    354            exp.ArrayContained: lambda self, e: self.binary(e, "<@"),
    355            exp.Merge: transforms.preprocess([transforms.remove_target_from_merge]),
    356            exp.Pivot: no_pivot_sql,
    357            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    358            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    359            exp.StrPosition: str_position_sql,
    360            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    361            exp.Substring: _substring_sql,
    362            exp.TimestampTrunc: timestamptrunc_sql,
    363            exp.TimeStrToTime: timestrtotime_sql,
    364            exp.TimeToStr: lambda self, e: f"TO_CHAR({self.sql(e, 'this')}, {self.format_time(e)})",
    365            exp.TableSample: no_tablesample_sql,
    366            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    367            exp.Trim: trim_sql,
    368            exp.TryCast: no_trycast_sql,
    369            exp.TsOrDsToDate: ts_or_ds_to_date_sql("postgres"),
    370            exp.UnixToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')})",
    371            exp.DataType: _datatype_sql,
    372            exp.GroupConcat: _string_agg_sql,
    373            exp.Array: lambda self, e: f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    374            if isinstance(seq_get(e.expressions, 0), exp.Select)
    375            else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]",
    376        }
    377
    378        PROPERTIES_LOCATION = {
    379            **generator.Generator.PROPERTIES_LOCATION,
    380            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    381            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    382        }
    383
    384        def bracket_sql(self, expression: exp.Bracket) -> str:
    385            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    386            if isinstance(expression.this, exp.Array):
    387                expression = expression.copy()
    388                expression.set("this", exp.paren(expression.this, copy=False))
    389
    390            return super().bracket_sql(expression)

DATE_DIFF_FACTOR = {'MICROSECOND': ' * 1000000', 'MILLISECOND': ' * 1000', 'SECOND': '', 'MINUTE': ' / 60', 'HOUR': ' / 3600', 'DAY': ' / 86400'}
class Postgres(sqlglot.dialects.dialect.Dialect): View Source

    187class Postgres(Dialect):
    188    INDEX_OFFSET = 1
    189    NULL_ORDERING = "nulls_are_large"
    190    TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
    191    TIME_MAPPING = {
    192        "AM": "%p",
    193        "PM": "%p",
    194        "D": "%u",  # 1-based day of week
    195        "DD": "%d",  # day of month
    196        "DDD": "%j",  # zero padded day of year
    197        "FMDD": "%-d",  # - is no leading zero for Python; same for FM in postgres
    198        "FMDDD": "%-j",  # day of year
    199        "FMHH12": "%-I",  # 9
    200        "FMHH24": "%-H",  # 9
    201        "FMMI": "%-M",  # Minute
    202        "FMMM": "%-m",  # 1
    203        "FMSS": "%-S",  # Second
    204        "HH12": "%I",  # 09
    205        "HH24": "%H",  # 09
    206        "MI": "%M",  # zero padded minute
    207        "MM": "%m",  # 01
    208        "OF": "%z",  # utc offset
    209        "SS": "%S",  # zero padded second
    210        "TMDay": "%A",  # TM is locale dependent
    211        "TMDy": "%a",
    212        "TMMon": "%b",  # Sep
    213        "TMMonth": "%B",  # September
    214        "TZ": "%Z",  # uppercase timezone name
    215        "US": "%f",  # zero padded microsecond
    216        "WW": "%U",  # 1-based week of year
    217        "YY": "%y",  # 15
    218        "YYYY": "%Y",  # 2015
    219    }
    220
    221    class Tokenizer(tokens.Tokenizer):
    222        QUOTES = ["'", "$$"]
    223
    224        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    225        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    226        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    227
    228        KEYWORDS = {
    229            **tokens.Tokenizer.KEYWORDS,
    230            "~~": TokenType.LIKE,
    231            "~~*": TokenType.ILIKE,
    232            "~*": TokenType.IRLIKE,
    233            "~": TokenType.RLIKE,
    234            "@>": TokenType.AT_GT,
    235            "<@": TokenType.LT_AT,
    236            "BEGIN": TokenType.COMMAND,
    237            "BEGIN TRANSACTION": TokenType.BEGIN,
    238            "BIGSERIAL": TokenType.BIGSERIAL,
    239            "CHARACTER VARYING": TokenType.VARCHAR,
    240            "DECLARE": TokenType.COMMAND,
    241            "DO": TokenType.COMMAND,
    242            "HSTORE": TokenType.HSTORE,
    243            "JSONB": TokenType.JSONB,
    244            "REFRESH": TokenType.COMMAND,
    245            "REINDEX": TokenType.COMMAND,
    246            "RESET": TokenType.COMMAND,
    247            "REVOKE": TokenType.COMMAND,
    248            "SERIAL": TokenType.SERIAL,
    249            "SMALLSERIAL": TokenType.SMALLSERIAL,
    250            "TEMP": TokenType.TEMPORARY,
    251            "CSTRING": TokenType.PSEUDO_TYPE,
    252        }
    253
    254        SINGLE_TOKENS = {
    255            **tokens.Tokenizer.SINGLE_TOKENS,
    256            "$": TokenType.PARAMETER,
    257        }
    258
    259        VAR_SINGLE_TOKENS = {"$"}
    260
    261    class Parser(parser.Parser):
    262        STRICT_CAST = False
    263        CONCAT_NULL_OUTPUTS_STRING = True
    264
    265        FUNCTIONS = {
    266            **parser.Parser.FUNCTIONS,
    267            "DATE_TRUNC": lambda args: exp.TimestampTrunc(
    268                this=seq_get(args, 1), unit=seq_get(args, 0)
    269            ),
    270            "GENERATE_SERIES": _generate_series,
    271            "NOW": exp.CurrentTimestamp.from_arg_list,
    272            "TO_CHAR": format_time_lambda(exp.TimeToStr, "postgres"),
    273            "TO_TIMESTAMP": _to_timestamp,
    274            "UNNEST": exp.Explode.from_arg_list,
    275        }
    276
    277        FUNCTION_PARSERS = {
    278            **parser.Parser.FUNCTION_PARSERS,
    279            "DATE_PART": lambda self: self._parse_date_part(),
    280        }
    281
    282        BITWISE = {
    283            **parser.Parser.BITWISE,
    284            TokenType.HASH: exp.BitwiseXor,
    285        }
    286
    287        EXPONENT = {
    288            TokenType.CARET: exp.Pow,
    289        }
    290
    291        RANGE_PARSERS = {
    292            **parser.Parser.RANGE_PARSERS,
    293            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    294            TokenType.AT_GT: binary_range_parser(exp.ArrayContains),
    295            TokenType.LT_AT: binary_range_parser(exp.ArrayContained),
    296        }
    297
    298        def _parse_factor(self) -> t.Optional[exp.Expression]:
    299            return self._parse_tokens(self._parse_exponent, self.FACTOR)
    300
    301        def _parse_exponent(self) -> t.Optional[exp.Expression]:
    302            return self._parse_tokens(self._parse_unary, self.EXPONENT)
    303
    304        def _parse_date_part(self) -> exp.Expression:
    305            part = self._parse_type()
    306            self._match(TokenType.COMMA)
    307            value = self._parse_bitwise()
    308
    309            if part and part.is_string:
    310                part = exp.var(part.name)
    311
    312            return self.expression(exp.Extract, this=part, expression=value)
    313
    314    class Generator(generator.Generator):
    315        SINGLE_STRING_INTERVAL = True
    316        LOCKING_READS_SUPPORTED = True
    317        JOIN_HINTS = False
    318        TABLE_HINTS = False
    319        PARAMETER_TOKEN = "$"
    320
    321        TYPE_MAPPING = {
    322            **generator.Generator.TYPE_MAPPING,
    323            exp.DataType.Type.TINYINT: "SMALLINT",
    324            exp.DataType.Type.FLOAT: "REAL",
    325            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    326            exp.DataType.Type.BINARY: "BYTEA",
    327            exp.DataType.Type.VARBINARY: "BYTEA",
    328            exp.DataType.Type.DATETIME: "TIMESTAMP",
    329        }
    330
    331        TRANSFORMS = {
    332            **generator.Generator.TRANSFORMS,
    333            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    334            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    335            exp.Explode: rename_func("UNNEST"),
    336            exp.JSONExtract: arrow_json_extract_sql,
    337            exp.JSONExtractScalar: arrow_json_extract_scalar_sql,
    338            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    339            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    340            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    341            exp.Pow: lambda self, e: self.binary(e, "^"),
    342            exp.CurrentDate: no_paren_current_date_sql,
    343            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    344            exp.DateAdd: _date_add_sql("+"),
    345            exp.DateStrToDate: datestrtodate_sql,
    346            exp.DateSub: _date_add_sql("-"),
    347            exp.DateDiff: _date_diff_sql,
    348            exp.LogicalOr: rename_func("BOOL_OR"),
    349            exp.LogicalAnd: rename_func("BOOL_AND"),
    350            exp.Max: max_or_greatest,
    351            exp.MapFromEntries: no_map_from_entries_sql,
    352            exp.Min: min_or_least,
    353            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    354            exp.ArrayContains: lambda self, e: self.binary(e, "@>"),
    355            exp.ArrayContained: lambda self, e: self.binary(e, "<@"),
    356            exp.Merge: transforms.preprocess([transforms.remove_target_from_merge]),
    357            exp.Pivot: no_pivot_sql,
    358            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    359            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    360            exp.StrPosition: str_position_sql,
    361            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    362            exp.Substring: _substring_sql,
    363            exp.TimestampTrunc: timestamptrunc_sql,
    364            exp.TimeStrToTime: timestrtotime_sql,
    365            exp.TimeToStr: lambda self, e: f"TO_CHAR({self.sql(e, 'this')}, {self.format_time(e)})",
    366            exp.TableSample: no_tablesample_sql,
    367            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    368            exp.Trim: trim_sql,
    369            exp.TryCast: no_trycast_sql,
    370            exp.TsOrDsToDate: ts_or_ds_to_date_sql("postgres"),
    371            exp.UnixToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')})",
    372            exp.DataType: _datatype_sql,
    373            exp.GroupConcat: _string_agg_sql,
    374            exp.Array: lambda self, e: f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    375            if isinstance(seq_get(e.expressions, 0), exp.Select)
    376            else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]",
    377        }
    378
    379        PROPERTIES_LOCATION = {
    380            **generator.Generator.PROPERTIES_LOCATION,
    381            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    382            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    383        }
    384
    385        def bracket_sql(self, expression: exp.Bracket) -> str:
    386            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    387            if isinstance(expression.this, exp.Array):
    388                expression = expression.copy()
    389                expression.set("this", exp.paren(expression.this, copy=False))
    390
    391            return super().bracket_sql(expression)

INDEX_OFFSET = 1
NULL_ORDERING = 'nulls_are_large'
TIME_FORMAT = "'YYYY-MM-DD HH24:MI:SS'"
TIME_MAPPING: Dict[str, str] = {'AM': '%p', 'PM': '%p', 'D': '%u', 'DD': '%d', 'DDD': '%j', 'FMDD': '%-d', 'FMDDD': '%-j', 'FMHH12': '%-I', 'FMHH24': '%-H', 'FMMI': '%-M', 'FMMM': '%-m', 'FMSS': '%-S', 'HH12': '%I', 'HH24': '%H', 'MI': '%M', 'MM': '%m', 'OF': '%z', 'SS': '%S', 'TMDay': '%A', 'TMDy': '%a', 'TMMon': '%b', 'TMMonth': '%B', 'TZ': '%Z', 'US': '%f', 'WW': '%U', 'YY': '%y', 'YYYY': '%Y'}
tokenizer_class = <class 'sqlglot.dialects.postgres.Postgres.Tokenizer'>
parser_class = <class 'sqlglot.dialects.postgres.Postgres.Parser'>
generator_class = <class 'sqlglot.dialects.postgres.Postgres.Generator'>
TIME_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}
FORMAT_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}
INVERSE_TIME_MAPPING: Dict[str, str] = {'%p': 'PM', '%u': 'D', '%d': 'DD', '%j': 'DDD', '%-d': 'FMDD', '%-j': 'FMDDD', '%-I': 'FMHH12', '%-H': 'FMHH24', '%-M': 'FMMI', '%-m': 'FMMM', '%-S': 'FMSS', '%I': 'HH12', '%H': 'HH24', '%M': 'MI', '%m': 'MM', '%z': 'OF', '%S': 'SS', '%A': 'TMDay', '%a': 'TMDy', '%b': 'TMMon', '%B': 'TMMonth', '%Z': 'TZ', '%f': 'US', '%U': 'WW', '%y': 'YY', '%Y': 'YYYY'}
INVERSE_TIME_TRIE: Dict = {'%': {'p': {0: True}, 'u': {0: True}, 'd': {0: True}, 'j': {0: True}, '-': {'d': {0: True}, 'j': {0: True}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'S': {0: True}}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'z': {0: True}, 'S': {0: True}, 'A': {0: True}, 'a': {0: True}, 'b': {0: True}, 'B': {0: True}, 'Z': {0: True}, 'f': {0: True}, 'U': {0: True}, 'y': {0: True}, 'Y': {0: True}}}
QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
BIT_START = "b'"
BIT_END = "'"
HEX_START = "x'"
HEX_END = "'"
BYTE_START = "e'"
BYTE_END = "'"
RAW_START = None
RAW_END = None

##### Inherited Members

sqlglot.dialects.dialect.Dialect
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  RESOLVES_IDENTIFIERS_AS_UPPERCASE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  STRICT_STRING_CONCAT
  NORMALIZE_FUNCTIONS
  DATE_FORMAT
  DATEINT_FORMAT
  FORMAT_MAPPING
  get_or_raise
  format_time
  normalize_identifier
  case_sensitive
  can_identify
  quote_identifier
  parse
  parse_into
  generate
  transpile
  tokenize
  tokenizer
  parser
  generator

class Postgres.Tokenizer(sqlglot.tokens.Tokenizer): View Source

    221    class Tokenizer(tokens.Tokenizer):
    222        QUOTES = ["'", "$$"]
    223
    224        BIT_STRINGS = [("b'", "'"), ("B'", "'")]
    225        HEX_STRINGS = [("x'", "'"), ("X'", "'")]
    226        BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
    227
    228        KEYWORDS = {
    229            **tokens.Tokenizer.KEYWORDS,
    230            "~~": TokenType.LIKE,
    231            "~~*": TokenType.ILIKE,
    232            "~*": TokenType.IRLIKE,
    233            "~": TokenType.RLIKE,
    234            "@>": TokenType.AT_GT,
    235            "<@": TokenType.LT_AT,
    236            "BEGIN": TokenType.COMMAND,
    237            "BEGIN TRANSACTION": TokenType.BEGIN,
    238            "BIGSERIAL": TokenType.BIGSERIAL,
    239            "CHARACTER VARYING": TokenType.VARCHAR,
    240            "DECLARE": TokenType.COMMAND,
    241            "DO": TokenType.COMMAND,
    242            "HSTORE": TokenType.HSTORE,
    243            "JSONB": TokenType.JSONB,
    244            "REFRESH": TokenType.COMMAND,
    245            "REINDEX": TokenType.COMMAND,
    246            "RESET": TokenType.COMMAND,
    247            "REVOKE": TokenType.COMMAND,
    248            "SERIAL": TokenType.SERIAL,
    249            "SMALLSERIAL": TokenType.SMALLSERIAL,
    250            "TEMP": TokenType.TEMPORARY,
    251            "CSTRING": TokenType.PSEUDO_TYPE,
    252        }
    253
    254        SINGLE_TOKENS = {
    255            **tokens.Tokenizer.SINGLE_TOKENS,
    256            "$": TokenType.PARAMETER,
    257        }
    258
    259        VAR_SINGLE_TOKENS = {"$"}

QUOTES = ["'", '$$']
BIT_STRINGS = [("b'", "'"), ("B'", "'")]
HEX_STRINGS = [("x'", "'"), ("X'", "'")]
BYTE_STRINGS = [("e'", "'"), ("E'", "'")]
KEYWORDS = {'{%': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{%-': <TokenType.BLOCK_START: 'BLOCK_START'>, '%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '+%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-%}': <TokenType.BLOCK_END: 'BLOCK_END'>, '{{+': <TokenType.BLOCK_START: 'BLOCK_START'>, '{{-': <TokenType.BLOCK_START: 'BLOCK_START'>, '+}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '-}}': <TokenType.BLOCK_END: 'BLOCK_END'>, '/*+': <TokenType.HINT: 'HINT'>, '==': <TokenType.EQ: 'EQ'>, '::': <TokenType.DCOLON: 'DCOLON'>, '||': <TokenType.DPIPE: 'DPIPE'>, '>=': <TokenType.GTE: 'GTE'>, '<=': <TokenType.LTE: 'LTE'>, '<>': <TokenType.NEQ: 'NEQ'>, '!=': <TokenType.NEQ: 'NEQ'>, '<=>': <TokenType.NULLSAFE_EQ: 'NULLSAFE_EQ'>, '->': <TokenType.ARROW: 'ARROW'>, '->>': <TokenType.DARROW: 'DARROW'>, '=>': <TokenType.FARROW: 'FARROW'>, '#>': <TokenType.HASH_ARROW: 'HASH_ARROW'>, '#>>': <TokenType.DHASH_ARROW: 'DHASH_ARROW'>, '<->': <TokenType.LR_ARROW: 'LR_ARROW'>, '&&': <TokenType.DAMP: 'DAMP'>, 'ALL': <TokenType.ALL: 'ALL'>, 'ALWAYS': <TokenType.ALWAYS: 'ALWAYS'>, 'AND': <TokenType.AND: 'AND'>, 'ANTI': <TokenType.ANTI: 'ANTI'>, 'ANY': <TokenType.ANY: 'ANY'>, 'ASC': <TokenType.ASC: 'ASC'>, 'AS': <TokenType.ALIAS: 'ALIAS'>, 'ASOF': <TokenType.ASOF: 'ASOF'>, 'AUTOINCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'AUTO_INCREMENT': <TokenType.AUTO_INCREMENT: 'AUTO_INCREMENT'>, 'BEGIN': <TokenType.COMMAND: 'COMMAND'>, 'BETWEEN': <TokenType.BETWEEN: 'BETWEEN'>, 'CACHE': <TokenType.CACHE: 'CACHE'>, 'UNCACHE': <TokenType.UNCACHE: 'UNCACHE'>, 'CASE': <TokenType.CASE: 'CASE'>, 'CHARACTER SET': <TokenType.CHARACTER_SET: 'CHARACTER_SET'>, 'CLUSTER BY': <TokenType.CLUSTER_BY: 'CLUSTER_BY'>, 'COLLATE': <TokenType.COLLATE: 'COLLATE'>, 'COLUMN': <TokenType.COLUMN: 'COLUMN'>, 'COMMIT': <TokenType.COMMIT: 'COMMIT'>, 'CONSTRAINT': <TokenType.CONSTRAINT: 'CONSTRAINT'>, 'CREATE': <TokenType.CREATE: 'CREATE'>, 'CROSS': <TokenType.CROSS: 'CROSS'>, 'CUBE': <TokenType.CUBE: 'CUBE'>, 'CURRENT_DATE': <TokenType.CURRENT_DATE: 'CURRENT_DATE'>, 'CURRENT_TIME': <TokenType.CURRENT_TIME: 'CURRENT_TIME'>, 'CURRENT_TIMESTAMP': <TokenType.CURRENT_TIMESTAMP: 'CURRENT_TIMESTAMP'>, 'CURRENT_USER': <TokenType.CURRENT_USER: 'CURRENT_USER'>, 'DATABASE': <TokenType.DATABASE: 'DATABASE'>, 'DEFAULT': <TokenType.DEFAULT: 'DEFAULT'>, 'DELETE': <TokenType.DELETE: 'DELETE'>, 'DESC': <TokenType.DESC: 'DESC'>, 'DESCRIBE': <TokenType.DESCRIBE: 'DESCRIBE'>, 'DISTINCT': <TokenType.DISTINCT: 'DISTINCT'>, 'DISTRIBUTE BY': <TokenType.DISTRIBUTE_BY: 'DISTRIBUTE_BY'>, 'DIV': <TokenType.DIV: 'DIV'>, 'DROP': <TokenType.DROP: 'DROP'>, 'ELSE': <TokenType.ELSE: 'ELSE'>, 'END': <TokenType.END: 'END'>, 'ESCAPE': <TokenType.ESCAPE: 'ESCAPE'>, 'EXCEPT': <TokenType.EXCEPT: 'EXCEPT'>, 'EXECUTE': <TokenType.EXECUTE: 'EXECUTE'>, 'EXISTS': <TokenType.EXISTS: 'EXISTS'>, 'FALSE': <TokenType.FALSE: 'FALSE'>, 'FETCH': <TokenType.FETCH: 'FETCH'>, 'FILTER': <TokenType.FILTER: 'FILTER'>, 'FIRST': <TokenType.FIRST: 'FIRST'>, 'FULL': <TokenType.FULL: 'FULL'>, 'FUNCTION': <TokenType.FUNCTION: 'FUNCTION'>, 'FOR': <TokenType.FOR: 'FOR'>, 'FOREIGN KEY': <TokenType.FOREIGN_KEY: 'FOREIGN_KEY'>, 'FORMAT': <TokenType.FORMAT: 'FORMAT'>, 'FROM': <TokenType.FROM: 'FROM'>, 'GEOGRAPHY': <TokenType.GEOGRAPHY: 'GEOGRAPHY'>, 'GEOMETRY': <TokenType.GEOMETRY: 'GEOMETRY'>, 'GLOB': <TokenType.GLOB: 'GLOB'>, 'GROUP BY': <TokenType.GROUP_BY: 'GROUP_BY'>, 'GROUPING SETS': <TokenType.GROUPING_SETS: 'GROUPING_SETS'>, 'HAVING': <TokenType.HAVING: 'HAVING'>, 'IF': <TokenType.IF: 'IF'>, 'ILIKE': <TokenType.ILIKE: 'ILIKE'>, 'IN': <TokenType.IN: 'IN'>, 'INDEX': <TokenType.INDEX: 'INDEX'>, 'INET': <TokenType.INET: 'INET'>, 'INNER': <TokenType.INNER: 'INNER'>, 'INSERT': <TokenType.INSERT: 'INSERT'>, 'INTERVAL': <TokenType.INTERVAL: 'INTERVAL'>, 'INTERSECT': <TokenType.INTERSECT: 'INTERSECT'>, 'INTO': <TokenType.INTO: 'INTO'>, 'IS': <TokenType.IS: 'IS'>, 'ISNULL': <TokenType.ISNULL: 'ISNULL'>, 'JOIN': <TokenType.JOIN: 'JOIN'>, 'KEEP': <TokenType.KEEP: 'KEEP'>, 'LATERAL': <TokenType.LATERAL: 'LATERAL'>, 'LEFT': <TokenType.LEFT: 'LEFT'>, 'LIKE': <TokenType.LIKE: 'LIKE'>, 'LIMIT': <TokenType.LIMIT: 'LIMIT'>, 'LOAD': <TokenType.LOAD: 'LOAD'>, 'LOCK': <TokenType.LOCK: 'LOCK'>, 'MERGE': <TokenType.MERGE: 'MERGE'>, 'NATURAL': <TokenType.NATURAL: 'NATURAL'>, 'NEXT': <TokenType.NEXT: 'NEXT'>, 'NEXT VALUE FOR': <TokenType.NEXT_VALUE_FOR: 'NEXT_VALUE_FOR'>, 'NOT': <TokenType.NOT: 'NOT'>, 'NOTNULL': <TokenType.NOTNULL: 'NOTNULL'>, 'NULL': <TokenType.NULL: 'NULL'>, 'OBJECT': <TokenType.OBJECT: 'OBJECT'>, 'OFFSET': <TokenType.OFFSET: 'OFFSET'>, 'ON': <TokenType.ON: 'ON'>, 'OR': <TokenType.OR: 'OR'>, 'ORDER BY': <TokenType.ORDER_BY: 'ORDER_BY'>, 'ORDINALITY': <TokenType.ORDINALITY: 'ORDINALITY'>, 'OUTER': <TokenType.OUTER: 'OUTER'>, 'OVER': <TokenType.OVER: 'OVER'>, 'OVERLAPS': <TokenType.OVERLAPS: 'OVERLAPS'>, 'OVERWRITE': <TokenType.OVERWRITE: 'OVERWRITE'>, 'PARTITION': <TokenType.PARTITION: 'PARTITION'>, 'PARTITION BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PARTITIONED_BY': <TokenType.PARTITION_BY: 'PARTITION_BY'>, 'PERCENT': <TokenType.PERCENT: 'PERCENT'>, 'PIVOT': <TokenType.PIVOT: 'PIVOT'>, 'PRAGMA': <TokenType.PRAGMA: 'PRAGMA'>, 'PRIMARY KEY': <TokenType.PRIMARY_KEY: 'PRIMARY_KEY'>, 'PROCEDURE': <TokenType.PROCEDURE: 'PROCEDURE'>, 'QUALIFY': <TokenType.QUALIFY: 'QUALIFY'>, 'RANGE': <TokenType.RANGE: 'RANGE'>, 'RECURSIVE': <TokenType.RECURSIVE: 'RECURSIVE'>, 'REGEXP': <TokenType.RLIKE: 'RLIKE'>, 'REPLACE': <TokenType.REPLACE: 'REPLACE'>, 'RETURNING': <TokenType.RETURNING: 'RETURNING'>, 'REFERENCES': <TokenType.REFERENCES: 'REFERENCES'>, 'RIGHT': <TokenType.RIGHT: 'RIGHT'>, 'RLIKE': <TokenType.RLIKE: 'RLIKE'>, 'ROLLBACK': <TokenType.ROLLBACK: 'ROLLBACK'>, 'ROLLUP': <TokenType.ROLLUP: 'ROLLUP'>, 'ROW': <TokenType.ROW: 'ROW'>, 'ROWS': <TokenType.ROWS: 'ROWS'>, 'SCHEMA': <TokenType.SCHEMA: 'SCHEMA'>, 'SELECT': <TokenType.SELECT: 'SELECT'>, 'SEMI': <TokenType.SEMI: 'SEMI'>, 'SET': <TokenType.SET: 'SET'>, 'SETTINGS': <TokenType.SETTINGS: 'SETTINGS'>, 'SHOW': <TokenType.SHOW: 'SHOW'>, 'SIMILAR TO': <TokenType.SIMILAR_TO: 'SIMILAR_TO'>, 'SOME': <TokenType.SOME: 'SOME'>, 'SORT BY': <TokenType.SORT_BY: 'SORT_BY'>, 'TABLE': <TokenType.TABLE: 'TABLE'>, 'TABLESAMPLE': <TokenType.TABLE_SAMPLE: 'TABLE_SAMPLE'>, 'TEMP': <TokenType.TEMPORARY: 'TEMPORARY'>, 'TEMPORARY': <TokenType.TEMPORARY: 'TEMPORARY'>, 'THEN': <TokenType.THEN: 'THEN'>, 'TRUE': <TokenType.TRUE: 'TRUE'>, 'UNION': <TokenType.UNION: 'UNION'>, 'UNNEST': <TokenType.UNNEST: 'UNNEST'>, 'UNPIVOT': <TokenType.UNPIVOT: 'UNPIVOT'>, 'UPDATE': <TokenType.UPDATE: 'UPDATE'>, 'USE': <TokenType.USE: 'USE'>, 'USING': <TokenType.USING: 'USING'>, 'UUID': <TokenType.UUID: 'UUID'>, 'VALUES': <TokenType.VALUES: 'VALUES'>, 'VIEW': <TokenType.VIEW: 'VIEW'>, 'VOLATILE': <TokenType.VOLATILE: 'VOLATILE'>, 'WHEN': <TokenType.WHEN: 'WHEN'>, 'WHERE': <TokenType.WHERE: 'WHERE'>, 'WINDOW': <TokenType.WINDOW: 'WINDOW'>, 'WITH': <TokenType.WITH: 'WITH'>, 'APPLY': <TokenType.APPLY: 'APPLY'>, 'ARRAY': <TokenType.ARRAY: 'ARRAY'>, 'BIT': <TokenType.BIT: 'BIT'>, 'BOOL': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BOOLEAN': <TokenType.BOOLEAN: 'BOOLEAN'>, 'BYTE': <TokenType.TINYINT: 'TINYINT'>, 'TINYINT': <TokenType.TINYINT: 'TINYINT'>, 'SHORT': <TokenType.SMALLINT: 'SMALLINT'>, 'SMALLINT': <TokenType.SMALLINT: 'SMALLINT'>, 'INT2': <TokenType.SMALLINT: 'SMALLINT'>, 'INTEGER': <TokenType.INT: 'INT'>, 'INT': <TokenType.INT: 'INT'>, 'INT4': <TokenType.INT: 'INT'>, 'LONG': <TokenType.BIGINT: 'BIGINT'>, 'BIGINT': <TokenType.BIGINT: 'BIGINT'>, 'INT8': <TokenType.BIGINT: 'BIGINT'>, 'DEC': <TokenType.DECIMAL: 'DECIMAL'>, 'DECIMAL': <TokenType.DECIMAL: 'DECIMAL'>, 'BIGDECIMAL': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'BIGNUMERIC': <TokenType.BIGDECIMAL: 'BIGDECIMAL'>, 'MAP': <TokenType.MAP: 'MAP'>, 'NULLABLE': <TokenType.NULLABLE: 'NULLABLE'>, 'NUMBER': <TokenType.DECIMAL: 'DECIMAL'>, 'NUMERIC': <TokenType.DECIMAL: 'DECIMAL'>, 'FIXED': <TokenType.DECIMAL: 'DECIMAL'>, 'REAL': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT4': <TokenType.FLOAT: 'FLOAT'>, 'FLOAT8': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE': <TokenType.DOUBLE: 'DOUBLE'>, 'DOUBLE PRECISION': <TokenType.DOUBLE: 'DOUBLE'>, 'JSON': <TokenType.JSON: 'JSON'>, 'CHAR': <TokenType.CHAR: 'CHAR'>, 'CHARACTER': <TokenType.CHAR: 'CHAR'>, 'NCHAR': <TokenType.NCHAR: 'NCHAR'>, 'VARCHAR': <TokenType.VARCHAR: 'VARCHAR'>, 'VARCHAR2': <TokenType.VARCHAR: 'VARCHAR'>, 'NVARCHAR': <TokenType.NVARCHAR: 'NVARCHAR'>, 'NVARCHAR2': <TokenType.NVARCHAR: 'NVARCHAR'>, 'STR': <TokenType.TEXT: 'TEXT'>, 'STRING': <TokenType.TEXT: 'TEXT'>, 'TEXT': <TokenType.TEXT: 'TEXT'>, 'CLOB': <TokenType.TEXT: 'TEXT'>, 'LONGVARCHAR': <TokenType.TEXT: 'TEXT'>, 'BINARY': <TokenType.BINARY: 'BINARY'>, 'BLOB': <TokenType.VARBINARY: 'VARBINARY'>, 'BYTEA': <TokenType.VARBINARY: 'VARBINARY'>, 'VARBINARY': <TokenType.VARBINARY: 'VARBINARY'>, 'TIME': <TokenType.TIME: 'TIME'>, 'TIMESTAMP': <TokenType.TIMESTAMP: 'TIMESTAMP'>, 'TIMESTAMPTZ': <TokenType.TIMESTAMPTZ: 'TIMESTAMPTZ'>, 'TIMESTAMPLTZ': <TokenType.TIMESTAMPLTZ: 'TIMESTAMPLTZ'>, 'DATE': <TokenType.DATE: 'DATE'>, 'DATETIME': <TokenType.DATETIME: 'DATETIME'>, 'INT4RANGE': <TokenType.INT4RANGE: 'INT4RANGE'>, 'INT4MULTIRANGE': <TokenType.INT4MULTIRANGE: 'INT4MULTIRANGE'>, 'INT8RANGE': <TokenType.INT8RANGE: 'INT8RANGE'>, 'INT8MULTIRANGE': <TokenType.INT8MULTIRANGE: 'INT8MULTIRANGE'>, 'NUMRANGE': <TokenType.NUMRANGE: 'NUMRANGE'>, 'NUMMULTIRANGE': <TokenType.NUMMULTIRANGE: 'NUMMULTIRANGE'>, 'TSRANGE': <TokenType.TSRANGE: 'TSRANGE'>, 'TSMULTIRANGE': <TokenType.TSMULTIRANGE: 'TSMULTIRANGE'>, 'TSTZRANGE': <TokenType.TSTZRANGE: 'TSTZRANGE'>, 'TSTZMULTIRANGE': <TokenType.TSTZMULTIRANGE: 'TSTZMULTIRANGE'>, 'DATERANGE': <TokenType.DATERANGE: 'DATERANGE'>, 'DATEMULTIRANGE': <TokenType.DATEMULTIRANGE: 'DATEMULTIRANGE'>, 'UNIQUE': <TokenType.UNIQUE: 'UNIQUE'>, 'STRUCT': <TokenType.STRUCT: 'STRUCT'>, 'VARIANT': <TokenType.VARIANT: 'VARIANT'>, 'ALTER': <TokenType.ALTER: 'ALTER'>, 'ANALYZE': <TokenType.COMMAND: 'COMMAND'>, 'CALL': <TokenType.COMMAND: 'COMMAND'>, 'COMMENT': <TokenType.COMMENT: 'COMMENT'>, 'COPY': <TokenType.COMMAND: 'COMMAND'>, 'EXPLAIN': <TokenType.COMMAND: 'COMMAND'>, 'GRANT': <TokenType.COMMAND: 'COMMAND'>, 'OPTIMIZE': <TokenType.COMMAND: 'COMMAND'>, 'PREPARE': <TokenType.COMMAND: 'COMMAND'>, 'TRUNCATE': <TokenType.COMMAND: 'COMMAND'>, 'VACUUM': <TokenType.COMMAND: 'COMMAND'>, 'USER-DEFINED': <TokenType.USERDEFINED: 'USERDEFINED'>, '~~': <TokenType.LIKE: 'LIKE'>, '~~*': <TokenType.ILIKE: 'ILIKE'>, '~*': <TokenType.IRLIKE: 'IRLIKE'>, '~': <TokenType.RLIKE: 'RLIKE'>, '@>': <TokenType.AT_GT: 'AT_GT'>, '<@': <TokenType.LT_AT: 'LT_AT'>, 'BEGIN TRANSACTION': <TokenType.BEGIN: 'BEGIN'>, 'BIGSERIAL': <TokenType.BIGSERIAL: 'BIGSERIAL'>, 'CHARACTER VARYING': <TokenType.VARCHAR: 'VARCHAR'>, 'DECLARE': <TokenType.COMMAND: 'COMMAND'>, 'DO': <TokenType.COMMAND: 'COMMAND'>, 'HSTORE': <TokenType.HSTORE: 'HSTORE'>, 'JSONB': <TokenType.JSONB: 'JSONB'>, 'REFRESH': <TokenType.COMMAND: 'COMMAND'>, 'REINDEX': <TokenType.COMMAND: 'COMMAND'>, 'RESET': <TokenType.COMMAND: 'COMMAND'>, 'REVOKE': <TokenType.COMMAND: 'COMMAND'>, 'SERIAL': <TokenType.SERIAL: 'SERIAL'>, 'SMALLSERIAL': <TokenType.SMALLSERIAL: 'SMALLSERIAL'>, 'CSTRING': <TokenType.PSEUDO_TYPE: 'PSEUDO_TYPE'>}
SINGLE_TOKENS = {'(': <TokenType.L_PAREN: 'L_PAREN'>, ')': <TokenType.R_PAREN: 'R_PAREN'>, '[': <TokenType.L_BRACKET: 'L_BRACKET'>, ']': <TokenType.R_BRACKET: 'R_BRACKET'>, '{': <TokenType.L_BRACE: 'L_BRACE'>, '}': <TokenType.R_BRACE: 'R_BRACE'>, '&': <TokenType.AMP: 'AMP'>, '^': <TokenType.CARET: 'CARET'>, ':': <TokenType.COLON: 'COLON'>, ',': <TokenType.COMMA: 'COMMA'>, '.': <TokenType.DOT: 'DOT'>, '-': <TokenType.DASH: 'DASH'>, '=': <TokenType.EQ: 'EQ'>, '>': <TokenType.GT: 'GT'>, '<': <TokenType.LT: 'LT'>, '%': <TokenType.MOD: 'MOD'>, '!': <TokenType.NOT: 'NOT'>, '|': <TokenType.PIPE: 'PIPE'>, '+': <TokenType.PLUS: 'PLUS'>, ';': <TokenType.SEMICOLON: 'SEMICOLON'>, '/': <TokenType.SLASH: 'SLASH'>, '\\': <TokenType.BACKSLASH: 'BACKSLASH'>, '*': <TokenType.STAR: 'STAR'>, '~': <TokenType.TILDA: 'TILDA'>, '?': <TokenType.PLACEHOLDER: 'PLACEHOLDER'>, '@': <TokenType.PARAMETER: 'PARAMETER'>, "'": <TokenType.QUOTE: 'QUOTE'>, '`': <TokenType.IDENTIFIER: 'IDENTIFIER'>, '"': <TokenType.IDENTIFIER: 'IDENTIFIER'>, '#': <TokenType.HASH: 'HASH'>, '$': <TokenType.PARAMETER: 'PARAMETER'>}
VAR_SINGLE_TOKENS = {'$'}

##### Inherited Members

sqlglot.tokens.Tokenizer
  RAW_STRINGS
  IDENTIFIERS
  IDENTIFIER_ESCAPES
  STRING_ESCAPES
  IDENTIFIERS_CAN_START_WITH_DIGIT
  WHITE_SPACE
  COMMANDS
  COMMAND_PREFIX_TOKENS
  NUMERIC_LITERALS
  ENCODE
  COMMENTS
  reset
  tokenize
  peek
  size
  sql
  tokens

class Postgres.Parser(sqlglot.parser.Parser): View Source

    261    class Parser(parser.Parser):
    262        STRICT_CAST = False
    263        CONCAT_NULL_OUTPUTS_STRING = True
    264
    265        FUNCTIONS = {
    266            **parser.Parser.FUNCTIONS,
    267            "DATE_TRUNC": lambda args: exp.TimestampTrunc(
    268                this=seq_get(args, 1), unit=seq_get(args, 0)
    269            ),
    270            "GENERATE_SERIES": _generate_series,
    271            "NOW": exp.CurrentTimestamp.from_arg_list,
    272            "TO_CHAR": format_time_lambda(exp.TimeToStr, "postgres"),
    273            "TO_TIMESTAMP": _to_timestamp,
    274            "UNNEST": exp.Explode.from_arg_list,
    275        }
    276
    277        FUNCTION_PARSERS = {
    278            **parser.Parser.FUNCTION_PARSERS,
    279            "DATE_PART": lambda self: self._parse_date_part(),
    280        }
    281
    282        BITWISE = {
    283            **parser.Parser.BITWISE,
    284            TokenType.HASH: exp.BitwiseXor,
    285        }
    286
    287        EXPONENT = {
    288            TokenType.CARET: exp.Pow,
    289        }
    290
    291        RANGE_PARSERS = {
    292            **parser.Parser.RANGE_PARSERS,
    293            TokenType.DAMP: binary_range_parser(exp.ArrayOverlaps),
    294            TokenType.AT_GT: binary_range_parser(exp.ArrayContains),
    295            TokenType.LT_AT: binary_range_parser(exp.ArrayContained),
    296        }
    297
    298        def _parse_factor(self) -> t.Optional[exp.Expression]:
    299            return self._parse_tokens(self._parse_exponent, self.FACTOR)
    300
    301        def _parse_exponent(self) -> t.Optional[exp.Expression]:
    302            return self._parse_tokens(self._parse_unary, self.EXPONENT)
    303
    304        def _parse_date_part(self) -> exp.Expression:
    305            part = self._parse_type()
    306            self._match(TokenType.COMMA)
    307            value = self._parse_bitwise()
    308
    309            if part and part.is_string:
    310                part = exp.var(part.name)
    311
    312            return self.expression(exp.Extract, this=part, expression=value)

Parser consumes a list of tokens produced by the Tokenizer and produces a parsed syntax tree.

###### Arguments:

- error_level: The desired error level. Default: ErrorLevel.IMMEDIATE
- error_message_context: Determines the amount of context to capture from a query string when displaying the error message (in number of characters). Default: 100
- max_errors: Maximum number of error messages to include in a raised ParseError. This is only relevant if error_level is ErrorLevel.RAISE. Default: 3

STRICT_CAST = False
CONCAT_NULL_OUTPUTS_STRING = True
FUNCTIONS = {'ABS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Abs'>>, 'ANY_VALUE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.AnyValue'>>, 'APPROX_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_COUNT_DISTINCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxDistinct'>>, 'APPROX_QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ApproxQuantile'>>, 'ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Array'>>, 'ARRAY_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAgg'>>, 'ARRAY_ALL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAll'>>, 'ARRAY_ANY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayAny'>>, 'ARRAY_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayConcat'>>, 'ARRAY_CONTAINS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayContains'>>, 'FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_FILTER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayFilter'>>, 'ARRAY_JOIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayJoin'>>, 'ARRAY_SIZE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySize'>>, 'ARRAY_SORT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySort'>>, 'ARRAY_SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArraySum'>>, 'ARRAY_UNION_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ArrayUnionAgg'>>, 'AVG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Avg'>>, 'CASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Case'>>, 'CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Cast'>>, 'CAST_TO_STR_TYPE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CastToStrType'>>, 'CEIL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'CEILING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ceil'>>, 'COALESCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'IFNULL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'NVL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Coalesce'>>, 'CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Concat'>>, 'CONCAT_WS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ConcatWs'>>, 'COUNT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Count'>>, 'COUNT_IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CountIf'>>, 'CURRENT_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDate'>>, 'CURRENT_DATETIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentDatetime'>>, 'CURRENT_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTime'>>, 'CURRENT_TIMESTAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'CURRENT_USER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentUser'>>, 'DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Date'>>, 'DATE_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateAdd'>>, 'DATEDIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATE_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateDiff'>>, 'DATEFROMPARTS': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateFromParts'>>, 'DATE_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateStrToDate'>>, 'DATE_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateSub'>>, 'DATE_TO_DATE_STR': <function Parser.<lambda>>, 'DATE_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DateToDi'>>, 'DATE_TRUNC': <function Postgres.Parser.<lambda>>, 'DATETIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeAdd'>>, 'DATETIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeDiff'>>, 'DATETIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeSub'>>, 'DATETIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DatetimeTrunc'>>, 'DAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Day'>>, 'DAY_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAYOFMONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfMonth'>>, 'DAY_OF_WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAYOFWEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfWeek'>>, 'DAY_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DAYOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DayOfYear'>>, 'DECODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Decode'>>, 'DI_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.DiToDate'>>, 'ENCODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Encode'>>, 'EXP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Exp'>>, 'EXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>, 'EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Extract'>>, 'FLOOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Floor'>>, 'FROM_BASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase'>>, 'FROM_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.FromBase64'>>, 'GENERATE_SERIES': <function _generate_series>, 'GREATEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Greatest'>>, 'GROUP_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.GroupConcat'>>, 'HEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hex'>>, 'HLL': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Hll'>>, 'IF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.If'>>, 'INITCAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Initcap'>>, 'JSONB_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtract'>>, 'JSONB_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONBExtractScalar'>>, 'JSON_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtract'>>, 'JSON_EXTRACT_SCALAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONExtractScalar'>>, 'JSON_FORMAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONFormat'>>, 'J_S_O_N_OBJECT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.JSONObject'>>, 'LAST_DATE_OF_MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LastDateOfMonth'>>, 'LEAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Least'>>, 'LEFT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Left'>>, 'LENGTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Length'>>, 'LEVENSHTEIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Levenshtein'>>, 'LN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Ln'>>, 'LOG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log'>>, 'LOG10': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log10'>>, 'LOG2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Log2'>>, 'LOGICAL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOL_AND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'BOOLAND_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalAnd'>>, 'LOGICAL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOL_OR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'BOOLOR_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.LogicalOr'>>, 'LOWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'LCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Lower'>>, 'MD5': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MD5'>>, 'MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Map'>>, 'MAP_FROM_ENTRIES': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MapFromEntries'>>, 'MATCH_AGAINST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.MatchAgainst'>>, 'MAX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Max'>>, 'MIN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Min'>>, 'MONTH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Month'>>, 'NEXT_VALUE_FOR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NextValueFor'>>, 'NUMBER_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.NumberToStr'>>, 'NVL2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Nvl2'>>, 'OPEN_J_S_O_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.OpenJSON'>>, 'PARAMETERIZED_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ParameterizedAgg'>>, 'PERCENTILE_CONT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileCont'>>, 'PERCENTILE_DISC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.PercentileDisc'>>, 'POSEXPLODE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Posexplode'>>, 'POWER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'POW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Pow'>>, 'QUANTILE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Quantile'>>, 'RANGE_N': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RangeN'>>, 'READ_CSV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ReadCSV'>>, 'REDUCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Reduce'>>, 'REGEXP_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpExtract'>>, 'REGEXP_I_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpILike'>>, 'REGEXP_LIKE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpLike'>>, 'REGEXP_SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RegexpSplit'>>, 'REPEAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Repeat'>>, 'RIGHT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Right'>>, 'ROUND': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Round'>>, 'ROW_NUMBER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.RowNumber'>>, 'SHA': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA1': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA'>>, 'SHA2': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SHA2'>>, 'SAFE_CONCAT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeConcat'>>, 'SAFE_DIVIDE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SafeDivide'>>, 'SET_AGG': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SetAgg'>>, 'SORT_ARRAY': <bound method Func.from_arg_list of <class 'sqlglot.expressions.SortArray'>>, 'SPLIT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Split'>>, 'SQRT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sqrt'>>, 'STANDARD_HASH': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StandardHash'>>, 'STAR_MAP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StarMap'>>, 'STDDEV': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Stddev'>>, 'STDDEV_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevPop'>>, 'STDDEV_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StddevSamp'>>, 'STR_POSITION': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrPosition'>>, 'STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToDate'>>, 'STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToTime'>>, 'STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StrToUnix'>>, 'STRUCT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Struct'>>, 'STRUCT_EXTRACT': <bound method Func.from_arg_list of <class 'sqlglot.expressions.StructExtract'>>, 'SUBSTRING': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Substring'>>, 'SUM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Sum'>>, 'TIME_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeAdd'>>, 'TIME_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeDiff'>>, 'TIME_STR_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToDate'>>, 'TIME_STR_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToTime'>>, 'TIME_STR_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeStrToUnix'>>, 'TIME_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeSub'>>, 'TIME_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToStr'>>, 'TIME_TO_TIME_STR': <function Parser.<lambda>>, 'TIME_TO_UNIX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeToUnix'>>, 'TIME_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimeTrunc'>>, 'TIMESTAMP_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampAdd'>>, 'TIMESTAMP_DIFF': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampDiff'>>, 'TIMESTAMP_SUB': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampSub'>>, 'TIMESTAMP_TRUNC': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TimestampTrunc'>>, 'TO_BASE64': <bound method Func.from_arg_list of <class 'sqlglot.expressions.ToBase64'>>, 'TO_CHAR': <function format_time_lambda.<locals>._format_time>, 'TRIM': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Trim'>>, 'TRY_CAST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TryCast'>>, 'TS_OR_DI_TO_DI': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDiToDi'>>, 'TS_OR_DS_ADD': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsAdd'>>, 'TS_OR_DS_TO_DATE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.TsOrDsToDate'>>, 'TS_OR_DS_TO_DATE_STR': <function Parser.<lambda>>, 'UNHEX': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Unhex'>>, 'UNIX_TO_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToStr'>>, 'UNIX_TO_TIME': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTime'>>, 'UNIX_TO_TIME_STR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.UnixToTimeStr'>>, 'UPPER': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'UCASE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Upper'>>, 'VAR_MAP': <function parse_var_map>, 'VARIANCE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VAR_SAMP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Variance'>>, 'VARIANCE_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'VAR_POP': <bound method Func.from_arg_list of <class 'sqlglot.expressions.VariancePop'>>, 'WEEK': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Week'>>, 'WEEK_OF_YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WEEKOFYEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.WeekOfYear'>>, 'WHEN': <bound method Func.from_arg_list of <class 'sqlglot.expressions.When'>>, 'X_M_L_TABLE': <bound method Func.from_arg_list of <class 'sqlglot.expressions.XMLTable'>>, 'YEAR': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Year'>>, 'GLOB': <function Parser.<lambda>>, 'LIKE': <function parse_like>, 'NOW': <bound method Func.from_arg_list of <class 'sqlglot.expressions.CurrentTimestamp'>>, 'TO_TIMESTAMP': <function _to_timestamp>, 'UNNEST': <bound method Func.from_arg_list of <class 'sqlglot.expressions.Explode'>>}
FUNCTION_PARSERS = {'ANY_VALUE': <function Parser.<lambda>>, 'CAST': <function Parser.<lambda>>, 'CONCAT': <function Parser.<lambda>>, 'CONVERT': <function Parser.<lambda>>, 'DECODE': <function Parser.<lambda>>, 'EXTRACT': <function Parser.<lambda>>, 'JSON_OBJECT': <function Parser.<lambda>>, 'LOG': <function Parser.<lambda>>, 'MATCH': <function Parser.<lambda>>, 'OPENJSON': <function Parser.<lambda>>, 'POSITION': <function Parser.<lambda>>, 'SAFE_CAST': <function Parser.<lambda>>, 'STRING_AGG': <function Parser.<lambda>>, 'SUBSTRING': <function Parser.<lambda>>, 'TRIM': <function Parser.<lambda>>, 'TRY_CAST': <function Parser.<lambda>>, 'TRY_CONVERT': <function Parser.<lambda>>, 'DATE_PART': <function Postgres.Parser.<lambda>>}
BITWISE = {<TokenType.AMP: 'AMP'>: <class 'sqlglot.expressions.BitwiseAnd'>, <TokenType.CARET: 'CARET'>: <class 'sqlglot.expressions.BitwiseXor'>, <TokenType.PIPE: 'PIPE'>: <class 'sqlglot.expressions.BitwiseOr'>, <TokenType.DPIPE: 'DPIPE'>: <class 'sqlglot.expressions.SafeDPipe'>, <TokenType.HASH: 'HASH'>: <class 'sqlglot.expressions.BitwiseXor'>}
EXPONENT = {<TokenType.CARET: 'CARET'>: <class 'sqlglot.expressions.Pow'>}
RANGE_PARSERS = {<TokenType.BETWEEN: 'BETWEEN'>: <function Parser.<lambda>>, <TokenType.GLOB: 'GLOB'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.ILIKE: 'ILIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IN: 'IN'>: <function Parser.<lambda>>, <TokenType.IRLIKE: 'IRLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.IS: 'IS'>: <function Parser.<lambda>>, <TokenType.LIKE: 'LIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.OVERLAPS: 'OVERLAPS'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.RLIKE: 'RLIKE'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.SIMILAR_TO: 'SIMILAR_TO'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.DAMP: 'DAMP'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.AT_GT: 'AT_GT'>: <function binary_range_parser.<locals>.<lambda>>, <TokenType.LT_AT: 'LT_AT'>: <function binary_range_parser.<locals>.<lambda>>}
INDEX_OFFSET: int = 1
NULL_ORDERING: str = 'nulls_are_large'
SHOW_TRIE: Dict = {}
SET_TRIE: Dict = {'GLOBAL': {0: True}, 'LOCAL': {0: True}, 'SESSION': {0: True}, 'TRANSACTION': {0: True}}
FORMAT_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}
TIME_MAPPING: Dict[str, str] = {'AM': '%p', 'PM': '%p', 'D': '%u', 'DD': '%d', 'DDD': '%j', 'FMDD': '%-d', 'FMDDD': '%-j', 'FMHH12': '%-I', 'FMHH24': '%-H', 'FMMI': '%-M', 'FMMM': '%-m', 'FMSS': '%-S', 'HH12': '%I', 'HH24': '%H', 'MI': '%M', 'MM': '%m', 'OF': '%z', 'SS': '%S', 'TMDay': '%A', 'TMDy': '%a', 'TMMon': '%b', 'TMMonth': '%B', 'TZ': '%Z', 'US': '%f', 'WW': '%U', 'YY': '%y', 'YYYY': '%Y'}
TIME_TRIE: Dict = {'A': {'M': {0: True}}, 'P': {'M': {0: True}}, 'D': {0: True, 'D': {0: True, 'D': {0: True}}}, 'F': {'M': {'D': {'D': {0: True, 'D': {0: True}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'S': {'S': {0: True}}}}, 'H': {'H': {'1': {'2': {0: True}}, '2': {'4': {0: True}}}}, 'M': {'I': {0: True}, 'M': {0: True}}, 'O': {'F': {0: True}}, 'S': {'S': {0: True}}, 'T': {'M': {'D': {'a': {'y': {0: True}}, 'y': {0: True}}, 'M': {'o': {'n': {0: True, 't': {'h': {0: True}}}}}}, 'Z': {0: True}}, 'U': {'S': {0: True}}, 'W': {'W': {0: True}}, 'Y': {'Y': {0: True, 'Y': {'Y': {0: True}}}}}

##### Inherited Members

sqlglot.parser.Parser
  Parser
  NO_PAREN_FUNCTIONS
  NESTED_TYPE_TOKENS
  ENUM_TYPE_TOKENS
  TYPE_TOKENS
  SUBQUERY_PREDICATES
  RESERVED_KEYWORDS
  DB_CREATABLES
  CREATABLES
  ID_VAR_TOKENS
  INTERVAL_VARS
  TABLE_ALIAS_TOKENS
  COMMENT_TABLE_ALIAS_TOKENS
  UPDATE_ALIAS_TOKENS
  TRIM_TYPES
  FUNC_TOKENS
  CONJUNCTION
  EQUALITY
  COMPARISON
  TERM
  FACTOR
  TIMESTAMPS
  SET_OPERATIONS
  JOIN_METHODS
  JOIN_SIDES
  JOIN_KINDS
  JOIN_HINTS
  LAMBDAS
  COLUMN_OPERATORS
  EXPRESSION_PARSERS
  STATEMENT_PARSERS
  UNARY_PARSERS
  PRIMARY_PARSERS
  PLACEHOLDER_PARSERS
  PROPERTY_PARSERS
  CONSTRAINT_PARSERS
  ALTER_PARSERS
  SCHEMA_UNNAMED_CONSTRAINTS
  NO_PAREN_FUNCTION_PARSERS
  FUNCTIONS_WITH_ALIASED_ARGS
  QUERY_MODIFIER_PARSERS
  SET_PARSERS
  SHOW_PARSERS
  TYPE_LITERAL_PARSERS
  MODIFIABLES
  DDL_SELECT_TOKENS
  PRE_VOLATILE_TOKENS
  TRANSACTION_KIND
  TRANSACTION_CHARACTERISTICS
  INSERT_ALTERNATIVES
  CLONE_KINDS
  TABLE_INDEX_HINT_TOKENS
  WINDOW_ALIAS_TOKENS
  WINDOW_BEFORE_PAREN_TOKENS
  WINDOW_SIDES
  ADD_CONSTRAINT_TOKENS
  PREFIXED_PIVOT_COLUMNS
  IDENTIFY_PIVOT_STRINGS
  LOG_BASE_FIRST
  LOG_DEFAULTS_TO_LN
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  STRICT_STRING_CONCAT
  FORMAT_MAPPING
  error_level
  error_message_context
  max_errors
  reset
  parse
  parse_into
  check_errors
  raise_error
  expression
  validate_expression
  errors
  sql

class Postgres.Generator(sqlglot.generator.Generator): View Source

    314    class Generator(generator.Generator):
    315        SINGLE_STRING_INTERVAL = True
    316        LOCKING_READS_SUPPORTED = True
    317        JOIN_HINTS = False
    318        TABLE_HINTS = False
    319        PARAMETER_TOKEN = "$"
    320
    321        TYPE_MAPPING = {
    322            **generator.Generator.TYPE_MAPPING,
    323            exp.DataType.Type.TINYINT: "SMALLINT",
    324            exp.DataType.Type.FLOAT: "REAL",
    325            exp.DataType.Type.DOUBLE: "DOUBLE PRECISION",
    326            exp.DataType.Type.BINARY: "BYTEA",
    327            exp.DataType.Type.VARBINARY: "BYTEA",
    328            exp.DataType.Type.DATETIME: "TIMESTAMP",
    329        }
    330
    331        TRANSFORMS = {
    332            **generator.Generator.TRANSFORMS,
    333            exp.BitwiseXor: lambda self, e: self.binary(e, "#"),
    334            exp.ColumnDef: transforms.preprocess([_auto_increment_to_serial, _serial_to_generated]),
    335            exp.Explode: rename_func("UNNEST"),
    336            exp.JSONExtract: arrow_json_extract_sql,
    337            exp.JSONExtractScalar: arrow_json_extract_scalar_sql,
    338            exp.JSONBExtract: lambda self, e: self.binary(e, "#>"),
    339            exp.JSONBExtractScalar: lambda self, e: self.binary(e, "#>>"),
    340            exp.JSONBContains: lambda self, e: self.binary(e, "?"),
    341            exp.Pow: lambda self, e: self.binary(e, "^"),
    342            exp.CurrentDate: no_paren_current_date_sql,
    343            exp.CurrentTimestamp: lambda *_: "CURRENT_TIMESTAMP",
    344            exp.DateAdd: _date_add_sql("+"),
    345            exp.DateStrToDate: datestrtodate_sql,
    346            exp.DateSub: _date_add_sql("-"),
    347            exp.DateDiff: _date_diff_sql,
    348            exp.LogicalOr: rename_func("BOOL_OR"),
    349            exp.LogicalAnd: rename_func("BOOL_AND"),
    350            exp.Max: max_or_greatest,
    351            exp.MapFromEntries: no_map_from_entries_sql,
    352            exp.Min: min_or_least,
    353            exp.ArrayOverlaps: lambda self, e: self.binary(e, "&&"),
    354            exp.ArrayContains: lambda self, e: self.binary(e, "@>"),
    355            exp.ArrayContained: lambda self, e: self.binary(e, "<@"),
    356            exp.Merge: transforms.preprocess([transforms.remove_target_from_merge]),
    357            exp.Pivot: no_pivot_sql,
    358            exp.RegexpLike: lambda self, e: self.binary(e, "~"),
    359            exp.RegexpILike: lambda self, e: self.binary(e, "~*"),
    360            exp.StrPosition: str_position_sql,
    361            exp.StrToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')}, {self.format_time(e)})",
    362            exp.Substring: _substring_sql,
    363            exp.TimestampTrunc: timestamptrunc_sql,
    364            exp.TimeStrToTime: timestrtotime_sql,
    365            exp.TimeToStr: lambda self, e: f"TO_CHAR({self.sql(e, 'this')}, {self.format_time(e)})",
    366            exp.TableSample: no_tablesample_sql,
    367            exp.ToChar: lambda self, e: self.function_fallback_sql(e),
    368            exp.Trim: trim_sql,
    369            exp.TryCast: no_trycast_sql,
    370            exp.TsOrDsToDate: ts_or_ds_to_date_sql("postgres"),
    371            exp.UnixToTime: lambda self, e: f"TO_TIMESTAMP({self.sql(e, 'this')})",
    372            exp.DataType: _datatype_sql,
    373            exp.GroupConcat: _string_agg_sql,
    374            exp.Array: lambda self, e: f"{self.normalize_func('ARRAY')}({self.sql(e.expressions[0])})"
    375            if isinstance(seq_get(e.expressions, 0), exp.Select)
    376            else f"{self.normalize_func('ARRAY')}[{self.expressions(e, flat=True)}]",
    377        }
    378
    379        PROPERTIES_LOCATION = {
    380            **generator.Generator.PROPERTIES_LOCATION,
    381            exp.TransientProperty: exp.Properties.Location.UNSUPPORTED,
    382            exp.VolatileProperty: exp.Properties.Location.UNSUPPORTED,
    383        }
    384
    385        def bracket_sql(self, expression: exp.Bracket) -> str:
    386            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    387            if isinstance(expression.this, exp.Array):
    388                expression = expression.copy()
    389                expression.set("this", exp.paren(expression.this, copy=False))
    390
    391            return super().bracket_sql(expression)

Generator converts a given syntax tree to the corresponding SQL string.

###### Arguments:

- pretty: Whether or not to format the produced SQL string. Default: False.
- identify: Determines when an identifier should be quoted. Possible values are: False (default): Never quote, except in cases where it's mandatory by the dialect. True or 'always': Always quote. 'safe': Only quote identifiers that are case insensitive.
- normalize: Whether or not to normalize identifiers to lowercase. Default: False.
- pad: Determines the pad size in a formatted string. Default: 2.
- indent: Determines the indentation size in a formatted string. Default: 2.
- normalize_functions: Whether or not to normalize all function names. Possible values are: "upper" or True (default): Convert names to uppercase. "lower": Convert names to lowercase. False: Disables function name normalization.
- unsupported_level: Determines the generator's behavior when it encounters unsupported expressions. Default ErrorLevel.WARN.
- max_unsupported: Maximum number of unsupported messages to include in a raised UnsupportedError. This is only relevant if unsupported_level is ErrorLevel.RAISE. Default: 3
- leading_comma: Determines whether or not the comma is leading or trailing in select expressions. This is only relevant when generating in pretty mode. Default: False
- max_text_width: The max number of characters in a segment before creating new lines in pretty mode. The default is on the smaller end because the length only represents a segment and not the true line length. Default: 80
- comments: Whether or not to preserve comments in the output SQL code. Default: True

SINGLE_STRING_INTERVAL = True
LOCKING_READS_SUPPORTED = True
JOIN_HINTS = False
TABLE_HINTS = False
PARAMETER_TOKEN = '$'
TYPE_MAPPING = {<Type.NCHAR: 'NCHAR'>: 'CHAR', <Type.NVARCHAR: 'NVARCHAR'>: 'VARCHAR', <Type.MEDIUMTEXT: 'MEDIUMTEXT'>: 'TEXT', <Type.LONGTEXT: 'LONGTEXT'>: 'TEXT', <Type.MEDIUMBLOB: 'MEDIUMBLOB'>: 'BLOB', <Type.LONGBLOB: 'LONGBLOB'>: 'BLOB', <Type.INET: 'INET'>: 'INET', <Type.TINYINT: 'TINYINT'>: 'SMALLINT', <Type.FLOAT: 'FLOAT'>: 'REAL', <Type.DOUBLE: 'DOUBLE'>: 'DOUBLE PRECISION', <Type.BINARY: 'BINARY'>: 'BYTEA', <Type.VARBINARY: 'VARBINARY'>: 'BYTEA', <Type.DATETIME: 'DATETIME'>: 'TIMESTAMP'}
TRANSFORMS = {<class 'sqlglot.expressions.DateAdd'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.TsOrDsAdd'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CaseSpecificColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CharacterSetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CheckColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CollateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.CommentColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DateFormatColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.DefaultColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.EncodeColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ExternalProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.InlineLengthColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LanguageProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LocationProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.LogProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.MaterializedProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnCommitProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.OnUpdateColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.PathColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ReturnsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SetProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SettingsProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.StabilityProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TemporaryProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.ToTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TransientProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.TitleColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.UppercaseColumnConstraint'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VarMap'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.VolatileProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <function Generator.<lambda>>, <class 'sqlglot.expressions.BitwiseXor'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ColumnDef'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Explode'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.JSONExtract'>: <function arrow_json_extract_sql>, <class 'sqlglot.expressions.JSONExtractScalar'>: <function arrow_json_extract_scalar_sql>, <class 'sqlglot.expressions.JSONBExtract'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.JSONBExtractScalar'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.JSONBContains'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Pow'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.CurrentDate'>: <function no_paren_current_date_sql>, <class 'sqlglot.expressions.CurrentTimestamp'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.DateStrToDate'>: <function datestrtodate_sql>, <class 'sqlglot.expressions.DateSub'>: <function _date_add_sql.<locals>.func>, <class 'sqlglot.expressions.DateDiff'>: <function _date_diff_sql>, <class 'sqlglot.expressions.LogicalOr'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.LogicalAnd'>: <function rename_func.<locals>.<lambda>>, <class 'sqlglot.expressions.Max'>: <function max_or_greatest>, <class 'sqlglot.expressions.MapFromEntries'>: <function no_map_from_entries_sql>, <class 'sqlglot.expressions.Min'>: <function min_or_least>, <class 'sqlglot.expressions.ArrayOverlaps'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayContains'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.ArrayContained'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Merge'>: <function preprocess.<locals>._to_sql>, <class 'sqlglot.expressions.Pivot'>: <function no_pivot_sql>, <class 'sqlglot.expressions.RegexpLike'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.RegexpILike'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.StrPosition'>: <function str_position_sql>, <class 'sqlglot.expressions.StrToTime'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Substring'>: <function _substring_sql>, <class 'sqlglot.expressions.TimestampTrunc'>: <function timestamptrunc_sql>, <class 'sqlglot.expressions.TimeStrToTime'>: <function timestrtotime_sql>, <class 'sqlglot.expressions.TimeToStr'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.TableSample'>: <function no_tablesample_sql>, <class 'sqlglot.expressions.ToChar'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.Trim'>: <function trim_sql>, <class 'sqlglot.expressions.TryCast'>: <function no_trycast_sql>, <class 'sqlglot.expressions.TsOrDsToDate'>: <function ts_or_ds_to_date_sql.<locals>._ts_or_ds_to_date_sql>, <class 'sqlglot.expressions.UnixToTime'>: <function Postgres.Generator.<lambda>>, <class 'sqlglot.expressions.DataType'>: <function _datatype_sql>, <class 'sqlglot.expressions.GroupConcat'>: <function _string_agg_sql>, <class 'sqlglot.expressions.Array'>: <function Postgres.Generator.<lambda>>}
PROPERTIES_LOCATION = {<class 'sqlglot.expressions.AlgorithmProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.AutoIncrementProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.BlockCompressionProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CharacterSetProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ChecksumProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.CollateProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.CopyGrantsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Cluster'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ClusteredByProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DataBlocksizeProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.DefinerProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.DictRange'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DictProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.DistStyleProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.EngineProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExecuteAsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.ExternalProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.FallbackProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.FileFormatProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.FreespaceProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.IsolatedLoadingProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.JournalProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.LanguageProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LikeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LocationProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.LockingProperty'>: <Location.POST_ALIAS: 'POST_ALIAS'>, <class 'sqlglot.expressions.LogProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.MaterializedProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.MergeBlockRatioProperty'>: <Location.POST_NAME: 'POST_NAME'>, <class 'sqlglot.expressions.NoPrimaryIndexProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.OnCommitProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.Order'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.PartitionedByProperty'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.PrimaryKey'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Property'>: <Location.POST_WITH: 'POST_WITH'>, <class 'sqlglot.expressions.ReturnsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatDelimitedProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.RowFormatSerdeProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SchemaCommentProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SerdeProperties'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.Set'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SettingsProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SetProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.SortKeyProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.SqlSecurityProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.StabilityProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TemporaryProperty'>: <Location.POST_CREATE: 'POST_CREATE'>, <class 'sqlglot.expressions.ToTableProperty'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.TransientProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.MergeTreeTTL'>: <Location.POST_SCHEMA: 'POST_SCHEMA'>, <class 'sqlglot.expressions.VolatileProperty'>: <Location.UNSUPPORTED: 'UNSUPPORTED'>, <class 'sqlglot.expressions.WithDataProperty'>: <Location.POST_EXPRESSION: 'POST_EXPRESSION'>, <class 'sqlglot.expressions.WithJournalTableProperty'>: <Location.POST_NAME: 'POST_NAME'>}
def bracket_sql(self, expression: sqlglot.expressions.Bracket) -> str: View Source

    385        def bracket_sql(self, expression: exp.Bracket) -> str:
    386            """Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY."""
    387            if isinstance(expression.this, exp.Array):
    388                expression = expression.copy()
    389                expression.set("this", exp.paren(expression.this, copy=False))
    390
    391            return super().bracket_sql(expression)

Forms like ARRAY[1, 2, 3][3] aren't allowed; we need to wrap the ARRAY.

SELECT_KINDS: Tuple[str, ...] = ()
INVERSE_TIME_MAPPING: Dict[str, str] = {'%p': 'PM', '%u': 'D', '%d': 'DD', '%j': 'DDD', '%-d': 'FMDD', '%-j': 'FMDDD', '%-I': 'FMHH12', '%-H': 'FMHH24', '%-M': 'FMMI', '%-m': 'FMMM', '%-S': 'FMSS', '%I': 'HH12', '%H': 'HH24', '%M': 'MI', '%m': 'MM', '%z': 'OF', '%S': 'SS', '%A': 'TMDay', '%a': 'TMDy', '%b': 'TMMon', '%B': 'TMMonth', '%Z': 'TZ', '%f': 'US', '%U': 'WW', '%y': 'YY', '%Y': 'YYYY'}
INVERSE_TIME_TRIE: Dict = {'%': {'p': {0: True}, 'u': {0: True}, 'd': {0: True}, 'j': {0: True}, '-': {'d': {0: True}, 'j': {0: True}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'S': {0: True}}, 'I': {0: True}, 'H': {0: True}, 'M': {0: True}, 'm': {0: True}, 'z': {0: True}, 'S': {0: True}, 'A': {0: True}, 'a': {0: True}, 'b': {0: True}, 'B': {0: True}, 'Z': {0: True}, 'f': {0: True}, 'U': {0: True}, 'y': {0: True}, 'Y': {0: True}}}
INDEX_OFFSET = 1
NULL_ORDERING = 'nulls_are_large'
@classmethod
def can_identify(text: str, identify: str | bool = 'safe') -> bool: View Source

    247    @classmethod
    248    def can_identify(cls, text: str, identify: str | bool = "safe") -> bool:
    249        """Checks if text can be identified given an identify option.
    250
    251        Args:
    252            text: The text to check.
    253            identify:
    254                "always" or `True`: Always returns true.
    255                "safe": True if the identifier is case-insensitive.
    256
    257        Returns:
    258            Whether or not the given text can be identified.
    259        """
    260        if identify is True or identify == "always":
    261            return True
    262
    263        if identify == "safe":
    264            return not cls.case_sensitive(text)
    265
    266        return False

Checks if text can be identified given an identify option.

###### Arguments:

- text: The text to check.
- identify: "always" or `True`: Always returns true. "safe": True if the identifier is case-insensitive.

###### Returns:

> Whether or not the given text can be identified.

QUOTE_START = "'"
QUOTE_END = "'"
IDENTIFIER_START = '"'
IDENTIFIER_END = '"'
STRING_ESCAPE = "'"
IDENTIFIER_ESCAPE = '"'
BIT_START: Optional[str] = "b'"
BIT_END: Optional[str] = "'"
HEX_START: Optional[str] = "x'"
HEX_END: Optional[str] = "'"
BYTE_START: Optional[str] = "e'"
BYTE_END: Optional[str] = "'"
RAW_START: Optional[str] = None
RAW_END: Optional[str] = None

##### Inherited Members

sqlglot.generator.Generator
  Generator
  NULL_ORDERING_SUPPORTED
  EXPLICIT_UNION
  WRAP_DERIVED_VALUES
  CREATE_FUNCTION_RETURN_AS
  MATCHED_BY_SOURCE
  INTERVAL_ALLOWS_PLURAL_FORM
  TABLESAMPLE_WITH_METHOD
  TABLESAMPLE_SIZE_IS_PERCENT
  LIMIT_FETCH
  RENAME_TABLE_WITH_DB
  GROUPINGS_SEP
  INDEX_ON
  IS_BOOL_ALLOWED
  STAR_MAPPING
  TIME_PART_SINGULARS
  TOKEN_MAPPING
  STRUCT_DELIMITER
  RESERVED_KEYWORDS
  WITH_SEPARATED_COMMENTS
  UNWRAPPED_INTERVAL_VALUES
  SENTINEL_LINE_BREAK
  UNNEST_COLUMN_ONLY
  ALIAS_POST_TABLESAMPLE
  IDENTIFIERS_CAN_START_WITH_DIGIT
  STRICT_STRING_CONCAT
  NORMALIZE_FUNCTIONS
  pretty
  identify
  normalize
  pad
  unsupported_level
  max_unsupported
  leading_comma
  max_text_width
  comments
  normalize_functions
  unsupported_messages
  generate
  unsupported
  sep
  seg
  pad_comment
  maybe_comment
  wrap
  no_identify
  normalize_func
  indent
  sql
  uncache_sql
  cache_sql
  characterset_sql
  column_sql
  columnposition_sql
  columndef_sql
  columnconstraint_sql
  autoincrementcolumnconstraint_sql
  compresscolumnconstraint_sql
  generatedasidentitycolumnconstraint_sql
  notnullcolumnconstraint_sql
  primarykeycolumnconstraint_sql
  uniquecolumnconstraint_sql
  createable_sql
  create_sql
  clone_sql
  describe_sql
  prepend_ctes
  with_sql
  cte_sql
  tablealias_sql
  bitstring_sql
  hexstring_sql
  bytestring_sql
  rawstring_sql
  datatypesize_sql
  datatype_sql
  directory_sql
  delete_sql
  drop_sql
  except_sql
  except_op
  fetch_sql
  filter_sql
  hint_sql
  index_sql
  identifier_sql
  inputoutputformat_sql
  national_sql
  partition_sql
  properties_sql
  root_properties
  properties
  with_properties
  locate_properties
  property_sql
  likeproperty_sql
  fallbackproperty_sql
  journalproperty_sql
  freespaceproperty_sql
  checksumproperty_sql
  mergeblockratioproperty_sql
  datablocksizeproperty_sql
  blockcompressionproperty_sql
  isolatedloadingproperty_sql
  lockingproperty_sql
  withdataproperty_sql
  insert_sql
  intersect_sql
  intersect_op
  introducer_sql
  pseudotype_sql
  onconflict_sql
  returning_sql
  rowformatdelimitedproperty_sql
  withtablehint_sql
  indextablehint_sql
  table_sql
  tablesample_sql
  pivot_sql
  tuple_sql
  update_sql
  values_sql
  var_sql
  into_sql
  from_sql
  group_sql
  having_sql
  join_sql
  lambda_sql
  lateral_sql
  limit_sql
  offset_sql
  setitem_sql
  set_sql
  pragma_sql
  lock_sql
  literal_sql
  escape_str
  loaddata_sql
  null_sql
  boolean_sql
  order_sql
  cluster_sql
  distribute_sql
  sort_sql
  ordered_sql
  matchrecognize_sql
  query_modifiers
  offset_limit_modifiers
  after_having_modifiers
  after_limit_modifiers
  select_sql
  schema_sql
  schema_columns_sql
  star_sql
  parameter_sql
  sessionparameter_sql
  placeholder_sql
  subquery_sql
  qualify_sql
  union_sql
  union_op
  unnest_sql
  where_sql
  window_sql
  partition_by_sql
  windowspec_sql
  withingroup_sql
  between_sql
  safebracket_sql
  all_sql
  any_sql
  exists_sql
  case_sql
  constraint_sql
  nextvaluefor_sql
  extract_sql
  trim_sql
  safeconcat_sql
  check_sql
  foreignkey_sql
  primarykey_sql
  if_sql
  matchagainst_sql
  jsonkeyvalue_sql
  jsonobject_sql
  openjsoncolumndef_sql
  openjson_sql
  in_sql
  in_unnest_op
  interval_sql
  return_sql
  reference_sql
  anonymous_sql
  paren_sql
  neg_sql
  not_sql
  alias_sql
  aliases_sql
  attimezone_sql
  add_sql
  and_sql
  connector_sql
  bitwiseand_sql
  bitwiseleftshift_sql
  bitwisenot_sql
  bitwiseor_sql
  bitwiserightshift_sql
  bitwisexor_sql
  cast_sql
  currentdate_sql
  collate_sql
  command_sql
  comment_sql
  mergetreettlaction_sql
  mergetreettl_sql
  transaction_sql
  commit_sql
  rollback_sql
  altercolumn_sql
  renametable_sql
  altertable_sql
  droppartition_sql
  addconstraint_sql
  distinct_sql
  ignorenulls_sql
  respectnulls_sql
  intdiv_sql
  dpipe_sql
  safedpipe_sql
  div_sql
  overlaps_sql
  distance_sql
  dot_sql
  eq_sql
  escape_sql
  glob_sql
  gt_sql
  gte_sql
  ilike_sql
  ilikeany_sql
  is_sql
  like_sql
  likeany_sql
  similarto_sql
  lt_sql
  lte_sql
  mod_sql
  mul_sql
  neq_sql
  nullsafeeq_sql
  nullsafeneq_sql
  or_sql
  slice_sql
  sub_sql
  trycast_sql
  use_sql
  binary
  function_fallback_sql
  func
  format_args
  text_width
  format_time
  expressions
  op_expressions
  naked_property
  set_operation
  tag_sql
  token_sql
  userdefinedfunction_sql
  joinhint_sql
  kwarg_sql
  when_sql
  merge_sql
  tochar_sql
  dictproperty_sql
  dictrange_sql
  dictsubproperty_sql
  oncluster_sql
  clusteredbyproperty_sql
  anyvalue_sql